/*
 * A page queue abstraction.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "page_queue.h"



PageQueue::~PageQueue()
{
}



void PageQueue::discard(Queue &queue, Memory *memory)
{
    while (!queue.empty())
    {
        Flexpage *flexpage = queue.front().flexpage;

        queue.pop_front();
        memory->release(flexpage->region);
        delete flexpage;
    }
}

bool PageQueue::remove(Queue &queue, Positions &positions, PageOwner *owner, Flexpage *flexpage)
{
    Positions::iterator position = positions.find(flexpage);

    if (position == positions.end())
        return false;

    /* The found owner may be different from the requesting owner or even NULL
       if another owner has acquired and then purged its pages. Such a purged
       flexpage is not immediately usable, however. */

    Queue::iterator entry = position->second;

    if ((entry->owner == NULL) || (entry->owner != owner))
        return false;

    queue.erase(entry);
    positions.erase(position);

    return true;
}

// vim: tabstop=4 expandtab shiftwidth=4
